@@ -1,3 +1,5 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+
 /* Metacity compositing manager */
 
 /* 
@@ -24,8 +26,18 @@
 
 #include "util.h"
 #include "display.h"
+#include "spring-model.h"
 
-typedef void (* MetaMinimizeFinishedFunc) (gpointer data);
+#ifdef HAVE_COMPOSITE_EXTENSIONS
+#include "cm/deform.h"
+/* FIXME: Needs namespacing. */
+void get_patch_points (Model *model, CmPoint points[4][4]);
+#endif
+
+typedef void (* MetaAnimationFinishedFunc) (gpointer data);
+
+/* XXX namespace me */
+void compute_window_rect (MetaWindow *window, MetaRectangle *rect);
 
 MetaCompositor* meta_compositor_new           (MetaDisplay       *display);
 void            meta_compositor_unref         (MetaCompositor    *compositor);
@@ -45,13 +57,36 @@ void meta_compositor_manage_screen   (MetaCompositor *compositor,
 void meta_compositor_unmanage_screen (MetaCompositor *compositor,
                                       MetaScreen     *screen);
 
-void meta_compositor_minimize (MetaCompositor           *compositor,
-			       MetaWindow               *window,
-			       int                       x,
-			       int                       y,
-			       int                       width,
-			       int                       height,
-			       MetaMinimizeFinishedFunc  finished_cb,
-			       gpointer                  finished_data);
+#if 0
+void meta_compositor_minimize (MetaCompositor            *compositor,
+			       MetaWindow                *window,
+			       int                        x,
+			       int                        y,
+			       int                        width,
+			       int                        height,
+			       MetaAnimationFinishedFunc  finished_cb,
+			       gpointer                   finished_data);
+#endif
+
+void
+meta_compositor_set_updates (MetaCompositor *compositor,
+			     MetaWindow *window,
+			     gboolean updates);
+void
+meta_compositor_destroy (MetaCompositor *compositor);
+
+void meta_compositor_begin_move (MetaCompositor *compositor,
+				 MetaWindow *window,
+				 MetaRectangle *initial,
+				 int grab_x, int grab_y);
+void meta_compositor_update_move (MetaCompositor *compositor,
+				  MetaWindow *window,
+				  int x, int y);
+void meta_compositor_end_move (MetaCompositor *compositor,
+			       MetaWindow *window);
+void meta_compositor_free_window (MetaCompositor *compositor,
+				  MetaWindow *window);
+void meta_compositor_free_window (MetaCompositor *compositor,
+				  MetaWindow *window);
 
 #endif /* META_COMPOSITOR_H */
