@@ -1,3 +1,5 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+
 /* MetaWindow property handling */
 
 /* 
@@ -22,6 +24,7 @@
 
 #include <config.h>
 #include "window-props.h"
+#include "errors.h"
 #include "xprops.h"
 #include "frame.h"
 #include "group.h"
@@ -248,9 +251,13 @@ set_title_text (MetaWindow  *window,
 
   /* Bug 330671 -- Don't forget to clear _NET_WM_VISIBLE_(ICON_)NAME */
   if (!modified && previous_was_modified)
-    XDeleteProperty (window->display->xdisplay,
-                     window->xwindow,
-                     atom);
+    {
+      meta_error_trap_push (window->display);
+      XDeleteProperty (window->display->xdisplay,
+                       window->xwindow,
+                       atom);
+      meta_error_trap_pop (window->display, FALSE);
+    }
 
   return modified;
 }
@@ -459,6 +466,8 @@ reload_net_wm_state (MetaWindow    *window,
         window->maximize_horizontally_after_placement = TRUE;
       else if (value->v.atom_list.atoms[i] == window->display->atom_net_wm_state_maximized_vert)
         window->maximize_vertically_after_placement = TRUE;
+      else if (value->v.atom_list.atoms[i] == window->display->atom_net_wm_state_hidden)
+        window->minimize_after_placement = TRUE;
       else if (value->v.atom_list.atoms[i] == window->display->atom_net_wm_state_modal)
         window->wm_state_modal = TRUE;
       else if (value->v.atom_list.atoms[i] == window->display->atom_net_wm_state_skip_taskbar)
@@ -703,14 +712,16 @@ reload_net_startup_id (MetaWindow    *window,
     window->initial_timestamp_set = 0;  
     window->initial_workspace_set = 0;
     
-    meta_screen_apply_startup_properties (window->screen, window);
+    if (meta_screen_apply_startup_properties (window->screen, window))
+      {
   
-    if (window->initial_timestamp_set)
-      timestamp = window->initial_timestamp;
-    if (window->initial_workspace_set)
-      workspace = meta_screen_get_workspace_by_index (window->screen, window->initial_workspace);
+        if (window->initial_timestamp_set)
+          timestamp = window->initial_timestamp;
+        if (window->initial_workspace_set)
+          workspace = meta_screen_get_workspace_by_index (window->screen, window->initial_workspace);
     
-    meta_window_activate_with_workspace (window, timestamp, workspace);
+        meta_window_activate_with_workspace (window, timestamp, workspace);
+      }
   }
   
   meta_verbose ("New _NET_STARTUP_ID \"%s\" for %s\n",
